/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKCONNECTIONHANDLER_H_
#define FANCYSOCKCONNECTIONHANDLER_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

class Runnable;

namespace Sock {

class Device;

/**
 * A factory interface for connection handlers.
 */
class FANCYSOCK_API ConnectionHandlerFactory: public virtual Fancy::Object {
protected:
	ConnectionHandlerFactory() {}
	virtual ~ConnectionHandlerFactory();

public:
	typedef Fancy::ObjPtr<ConnectionHandlerFactory> ptr_t;

	/**
	 * A thread local key under which metadata like authentication infos
     * or HTTP headers are stored as a Map by connection handlers.
	 * 
	 * Call
	 * 
	 *  Map:ptr_t meta = Thread::getSpecific(ConnectionHandlerFactory::metadataThreadLocalKey);
	 * in order to get the stored metadata.
	 */
	static Fancy::Object::ptr_t metadataThreadLocalKey;

	/**
	 * Instantiate a new connection handler for the given device.
	 *
	 * @param device The device to mange in side the returned runnable.
	 * @return A new connection handler instance suitable for the given device.
	 */
	virtual Fancy::ObjPtr<Fancy::Runnable> newConnectionHandler(const Fancy::ObjPtr<Device> &device) = 0;
};

}
}

#endif /* FANCYSOCKCONNECTIONHANDLER_H_ */
