/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKINVOCATIONHANDLER_H_
#define FANCYSOCKINVOCATIONHANDLER_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

class Map;
template <class T> class ObjectArray;

namespace Sock {

class InvocationContext;

/**
 * An invocation handler, which is registered under a subsystem name at a server
 * or handles callbacks on a client.
 */
class FANCYSOCK_API InvocationHandler: public virtual Fancy::Object {
protected:
	InvocationHandler();
	virtual ~InvocationHandler();
public:
	typedef Fancy::ObjPtr<InvocationHandler> ptr_t;

	/**
 	 * @param ctx The context under which the call has been issued.
	 * @param arg The argument of the call.
	 * @param metadata The metadata accompanying the remote call, which is passed to the
	 *                 invocation handler. Metadata may be used to communicate authorization
	 *                 information such as user/password or a session Id.
	 * @return The result of the operation.
	 */
	virtual Object::ptr_t invoke(const Fancy::ObjPtr<InvocationContext> &ctx,
			                     const Fancy::ObjPtr<Fancy::Object> &arg,
			                     const Fancy::ObjPtr<Fancy::Map> &metadata) = 0;
};

}

}

#endif /* FANCYSOCKINVOCATIONHANDLER_H_ */
