/***********************************************************
 * $Id$
 *
 * FancySock HTTPD helpers
 *
 * Copyright (C) 2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKJSONRPC2REQUESTHANDLER_H_
#define FANCYSOCKJSONRPC2REQUESTHANDLER_H_

#include <fancysockhttpdrequesthandler.h>

namespace Fancy {

class ReflectionTarget;
class SerializableFactory;
class TypeConverter;
class String;

namespace Sock {
namespace HttpD {

class SessionManager;

class FANCYSOCK_API JsonRpc2RequestHandler : virtual public HttpDRequestHandler {

protected:
	JsonRpc2RequestHandler();
	virtual ~JsonRpc2RequestHandler();
public:

	typedef ObjPtr<JsonRpc2RequestHandler> ptr_t;

	static ptr_t newInstance(const std::string &servicePath,
			const ObjPtr<SerializableFactory> &sf,
			const ObjPtr<ReflectionTarget> &target);

	static ptr_t newInstance(const std::string &servicePath,
			const ObjPtr<SerializableFactory> &sf,
			const ObjPtr<TypeConverter> &tc,
			const ObjPtr<ReflectionTarget> &target);

	template<class Interface>
	static ptr_t newInstanceT(const std::string &servicePath,
			const ObjPtr<SerializableFactory> &sf,
			const ObjPtr<Interface> &target) {
		return newInstance(servicePath,sf,Interface::reflect(target));
	}

	template<class Interface>
	static ptr_t newInstanceT(const std::string &servicePath,
			const ObjPtr<SerializableFactory> &sf,
			const ObjPtr<TypeConverter> &tc,
			const ObjPtr<Interface> &target) {
		return newInstance(servicePath,sf,tc,Interface::reflect(target));
	}

	virtual ObjPtr<ReflectionTarget> getTarget() const = 0;

	/**
	 * @return The service path under which the client application
	 *         is advised to send JSON-RPC POST requests.
	 *         This should be set to the
	 *         path under which this request handler is registered.
	 */
	virtual std::string getServicePath() const = 0;

	/**
	 * @return An optional session manager, which is set, if calls to this service need an
	 *         authenticated user in the HTTP session.
	 */
	virtual ObjPtr<SessionManager> getSessionManager() const = 0;

	/**
	 * @return An optional thread local key under which the name of the authenticated
	 *         user is bound to the current thread.
	 */
	virtual ObjPtr<Object> getLoginThreadLocalKey() const = 0;

	/**
	 * Set an optional session manager, which is non-null, if only authenticated users
	 * are allowed to call this service.
	 */
	virtual void setSessionManager(const ObjPtr<SessionManager> &sessionManager) = 0;

	/**
	 * Set the optional thread local key under which the name of the authenticated
	 *         user is bound to the current thread.
	 */
	virtual void setLoginThreadLocalKey(const ObjPtr<Object> &key) = 0;

};

} /* namespace HttpD */
} /* namespace Sock */
} /* namespace Fancy */

#endif /* FANCYSOCKJSONRPC2REQUESTHANDLER_H_ */
