/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Global logging stuff.
** 
*****************************************************************************/

#ifndef FANCYSOCK_LOG_H
#define FANCYSOCK_LOG_H

#include <fancylog.h>
#include <fancysock.h>

// facility masks:
// facility "fancysock"
#define  FANCYSOCK_LOG_BASE      (1<<0)
// facility "fancytrans"
#define  FANCYSOCK_LOG_TRANSPORT (1<<1)
// facility "fancyloop"
#define  FANCYSOCK_LOG_LOOP      (1<<2)
// facility "fancyserver"
#define  FANCYSOCK_LOG_SERVER    (1<<3)
// facility "fancyremoting"
#define  FANCYSOCK_LOG_REMOTING  (1<<4)
// facility "fancydaemon"
#define  FANCYSOCK_LOG_DAEMON    (1<<5)

namespace Fancy {
  namespace Sock {

    extern FANCYSOCK_API fancylog_service *log_service;

    // parse for logging cmdline args and filter out recognized options.
    FANCYSOCK_API void fancylog_parse_arg (int *argc, char **argv);

    inline void vfancylog (int facility,
                           int priority, const char *fmt, va_list ap)
      {
        va_list aq;
        va_copy(aq,ap);
        ::vfancylog(log_service,facility,priority,fmt,aq);
        va_end(aq);
      }

#ifdef __GNUG__
  inline void fancylog (int facility,
                        int priority, const char *fmt, ...)
    __attribute__((format(printf,3,4)));
    
#endif

    inline void fancylog (int facility,
                          int priority, const char *fmt, ...)
      {
        va_list ap;
        va_start(ap, fmt);
        vfancylog(facility,priority,fmt,ap);
        va_end(ap);
      }
  }
}

#endif // ! FANCYSOCK_LOG_H
