/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / ITEG
**
** A simple NTP client.
**
*****************************************************************************/

#ifndef FANCYSOCK_NTP_CLIENT_H
#define FANCYSOCK_NTP_CLIENT_H

#include <fancyobject.h>
#include <fancysock.h>

namespace Fancy {
namespace Sock {

class Address;
class NtpStatus;

/**
 * A simple NTP client
 */
class FANCYSOCK_API NtpClient : virtual public Fancy::Object
{
public:
    typedef Fancy::ObjPtr<NtpClient> ptr_t;

    /**
     * Instantiate a new NTP client instance
     *
     * This method will use close_on_exec=true
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Address> &server, int timeoutMillis);

    virtual Fancy::ObjPtr<Address> getServerAddress() const = 0;

    virtual int getTimeoutMillis() const = 0;

    /**
     * Send an NTP query to the given address.
     */
    virtual Fancy::ObjPtr<NtpStatus> queryStatus() = 0;

  protected:
	  NtpClient();
	  virtual ~NtpClient();
  };


  }
}

#endif // ! FANCYSOCK_PACKET_DEVICE_H
