/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKONEWAYHANDLER_H_
#define FANCYSOCKONEWAYHANDLER_H_

#include <fancyobject.h>
#include <fancysock.h>

namespace Fancy {

class Map;

namespace Sock {

class InvocationContext;

/**
 * A handler for oneway invocations which is registered under a subsystem name at a client.
 */
class FANCYSOCK_API OneWayHandler: public virtual Fancy::Object {
protected:
	OneWayHandler();
	virtual ~OneWayHandler();
public:
	typedef Fancy::ObjPtr<OneWayHandler> ptr_t;

    /**
     * Handle a oneway invocation. No result is reported back to the server.
     *
     * @param ctx The context under which the call has been issued.
     * @param arg The argument of the call.
     * @param metadata The metadata accompanying the remote call, which is passed to the
     *                 invocation handler. Metadata may be used to communicate authorization
     *                 information such as user/password or a session Id.
     */
	virtual void doOneWay (const Fancy::ObjPtr<InvocationContext> &ctx,
			               const Object::ptr_t &arg,
			               const Fancy::ObjPtr<Map> &metadata) = 0;
};

}

}

#endif /* FANCYSOCKONEWAYHANDLER_H_ */
