/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKPING_H_
#define FANCYSOCKPING_H_

#include <fancysock.h>

namespace Fancy {

template <class T> class ObjPtr;

namespace Sock {

class Address;

/**
 * Check for the existance of a network node by using the operating system's ping command.
 *
 * @param addr The host to ping. The port of the address will be ignored.
 * @param maxhops The maximal number of hops of the packet. This value traditionally has been called 'time to live',
 *                but it essentially limits the hops count of the packet to travel. PLease be aware that the
 *                hop count includes the number of hops from your node to the remote host and the number of host
 *                from the remote host back to your node.
 * @param timeout The maximal number of milliseconds the ping operation will last.
 * @param npings The number of packets to send for probing the remote host.
 */
extern bool FANCYSOCK_API ping(const Fancy::ObjPtr<Address> &addr, int maxhops=63, int timeout=2000, int npings=1);

}}

#endif /* FANCYSOCKPING_H_ */
