/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKREMOTINGCLIENT_H_
#define FANCYSOCKREMOTINGCLIENT_H_

#include <fancysockinvocationcontext.h>
#include <fancysockresolveoptions.h>

#include <fancypoolableconnection.h>

namespace Fancy {

class Map;
class SerializableFactory;
class String;
class ThreadPool;
class UID;

namespace Sock {

class InvocationHandler;
class OneWayHandler;
class DeviceFactory;

class FANCYSOCK_API RemotingClient: public virtual InvocationContext, public virtual Fancy::PoolableConnection {
protected:
	RemotingClient();
	virtual ~RemotingClient();

public:
	typedef Fancy::ObjPtr<RemotingClient> ptr_t;

	static ptr_t newInstance(const std::string &url,
			                 const Fancy::ObjPtr<DeviceFactory> &deviceFactory,
			                 const Fancy::ObjPtr<Fancy::SerializableFactory> &serializableFactory,
			                 const Fancy::ObjPtr<ResolveOptions>& resolveOptions=0);

	static ptr_t newInstance(const std::string &url,
			                 const Fancy::ObjPtr<DeviceFactory> &deviceFactory,
			                 const Fancy::ObjPtr<Fancy::SerializableFactory> &serializableFactory,
			                 const Fancy::ObjPtr<Fancy::ThreadPool> &threadPool,
			                 const Fancy::ObjPtr<ResolveOptions>& resolveOptions=0);

	virtual Fancy::Object::ptr_t invoke(const Fancy::ObjPtr<Fancy::String> &subSystem,
			                            const Fancy::Object::ptr_t &arg,
			                            const Fancy::ObjPtr<Fancy::Map> &metadata) = 0;

	virtual Fancy::ObjPtr<Fancy::UID> registerCallback(const Fancy::ObjPtr<InvocationHandler> &handler) = 0;

	virtual void deregisterCallback(const Fancy::ObjPtr<Fancy::UID> &handlerUID) = 0;

	virtual Fancy::ObjPtr<InvocationHandler> getCallback(const Fancy::ObjPtr<Fancy::UID> &handlerUID) const = 0;

	virtual void registerBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem,
			                              const Fancy::ObjPtr<OneWayHandler> &handler) = 0;

	virtual void deregisterBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) = 0;

	virtual Fancy::ObjPtr<OneWayHandler> getBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) const = 0;

	/**
	 * @return The buffer size used to block data on the underlying socket.
	 */
	virtual int getBufferSize() const = 0;

	/**
	 * @param n The buffer size used to block data on the underlying socket to set.
	 *          The default value is @c 1460, which fits best into a normal TCP/IP over
	 *          ethernet connection with a MTU of 1500 bytes.
	 */
    virtual void setBufferSize(int n) = 0;
};

}

}

#endif /* FANCYSOCKREMOTINGCLIENT_H_ */
