/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKREMOTINGCLIENTPOOL_H_
#define FANCYSOCKREMOTINGCLIENTPOOL_H_

#define FANCYSOCK_REMOTINGCLIENTPOOL_BEAN "Fancy::Sock::RemotingClientPool"

#include <fancysockinvocationcontext.h>

namespace Fancy {

class SerializableFactory;
class String;
class ThreadPool;

namespace Sock {

class InvocationHandler;
class OneWayHandler;
class DeviceFactory;
class RemotingClient;

class FANCYSOCK_API RemotingClientPool: public virtual Object {
protected:
	RemotingClientPool();
	virtual ~RemotingClientPool();

public:
	typedef Fancy::ObjPtr<RemotingClientPool> ptr_t;

	static ptr_t newInstance(size_t minSize, size_t maxSize,
	                         unsigned idleTimeout, unsigned waitTimeout,
	                         const std::string &url,
			                 const Fancy::ObjPtr<DeviceFactory> &deviceFactory,
			                 const Fancy::ObjPtr<Fancy::SerializableFactory> &serializableFactory);

	static ptr_t newInstance(size_t minSize, size_t maxSize,
                             unsigned idleTimeout, unsigned waitTimeout,
                             const std::string &url,
			                 const Fancy::ObjPtr<DeviceFactory> &deviceFactory,
			                 const Fancy::ObjPtr<Fancy::SerializableFactory> &serializableFactory,
			                 const Fancy::ObjPtr<Fancy::ThreadPool> &threadPool);

	/** must be called before start() */
	virtual void registerBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem,
			                              const Fancy::ObjPtr<OneWayHandler> &handler) = 0;

	virtual void deregisterBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) = 0;

	virtual Fancy::ObjPtr<OneWayHandler> getBroadcastHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) const = 0;

    virtual int getBufferSize() const = 0;
    virtual void setBufferSize(int n) = 0;

    virtual Fancy::ObjPtr<RemotingClient> takeClient() = 0;

    virtual void putBackClient(const Fancy::ObjPtr<RemotingClient> &client) = 0;

    virtual void start() = 0;
    virtual void stop() = 0;
};

}

}

#endif /* FANCYSOCKREMOTINGCLIENTPOOL_H_ */
