/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKSESSIONMANAGER_H_
#define FANCYSOCKSESSIONMANAGER_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

class Map;
class UID;

namespace Sock {

/**
 * A session manager is used to maintain a server-side, per-client session.
 * The session information may be attached to the current thread or stored
 * in a per-client map, whatever suits best to your rquirements.
 *
 * Most likely, your @c SessionManager implementation will play together with
 * an @c InvocationAuthenticator implementation. It also possible to implement
 * both interfaces, @c SessionManager and @c InvocationAuthenticator in one
 * bean.
 *
 * For the HTTPD side of things look at
 * Fancy::Sock::HttpD::SessionManager in fancysockhttpdsession.h
 */
class FANCYSOCK_API SessionManager: public virtual Fancy::Object {
protected:
    SessionManager();
    virtual ~SessionManager();
public:
    typedef Fancy::ObjPtr<SessionManager> ptr_t;

    /**
     * Open a session and attach it to the current thread.
     *
     * @param clientUID The client UID for which to open a session.
     * @param metadata The request metadata as passed to
     *                 @c RemotingBeanImport::setMetadata().
     */
    virtual void openSession(const Fancy::ObjPtr<Fancy::UID> &clientUID,
                             const Fancy::ObjPtr<Fancy::Map> metadata) = 0;

    /**
     * Close a session and withdraw it from the current thread.
     *
     * @param sessionId The client UID of which to close a session previously
     *                  created by @c openSession.
     */
    virtual void closeSession(const Fancy::ObjPtr<Fancy::UID> &clientUID) = 0;
};

}

}

#endif /* FANCYSOCKSESSIONMANAGER_H_ */
