/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** stream classes, which encapsulate writing/reading a document from/to a
** transport.
** 
*****************************************************************************/

#ifndef FANCYSOCK_TRANS_STREAM_H
#define FANCYSOCK_TRANS_STREAM_H

#include <iostream>
#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {
class Map;

namespace Sock {

class Transport;

class FANCYSOCK_API itransstreambuf : public std::streambuf
{
public:
    /**
     * Wrap a transport as input stream buffer and call
     * t->openReadDoc(headers,timeout) in the constructor.
     */
    itransstreambuf(const Fancy::ObjPtr<Transport> &t, Fancy::ObjPtr<Map> &headers, int timeout);
    itransstreambuf(const Fancy::ObjPtr<Transport> &t);
    /**
     * Wrap a transport as input stream buffer and do nothing in the constructor.
     *
     * It is assumed, that t->openReadDoc(headers,timeout) has been called before.
     */
    virtual ~itransstreambuf();

protected:
    int_type underflow ();

private:
    int fill_buffer();

    Fancy::ObjPtr<Transport> transport;
};

class FANCYSOCK_API itransstream : public std::istream
{
    itransstreambuf itsbuf;
public:
    /**
     * Wrap a transport as input stream and call
     * t->openReadDoc(headers,timeout) in the constructor.
     */
    itransstream(const Fancy::ObjPtr<Transport> &t, Fancy::ObjPtr<Map> &headers, int timeout);
    /**
     * Wrap a transport as input stream and do nothing in the constructor.
     *
     * It is assumed, that t->openReadDoc(headers,timeout) has been called before.
     */
    itransstream(const Fancy::ObjPtr<Transport> &t);
    virtual ~itransstream();
};

class FANCYSOCK_API otransstreambuf : public std::streambuf
{
public:
    /**
     * Wrap a transport as output stream buffer and call
     * t->openWriteDoc(headers) in the constructor.
     */
    otransstreambuf(const Fancy::ObjPtr<Transport> &t, const Fancy::ObjPtr<Map> &headers);
    /**
     * Wrap a transport as output stream buffer and do nothing in the constructor.
     *
     * It is assumed, that t->openWriteDoc(headers) has been called before.
     */
    otransstreambuf(const Fancy::ObjPtr<Transport> &t);
    virtual ~otransstreambuf();

protected:
    int_type overflow (int_type c);
    int sync();

private:
    int put_buffer();

    Fancy::ObjPtr<Transport> transport;
};

class FANCYSOCK_API otransstream : public std::ostream
{
    otransstreambuf otsbuf;
public:
    /**
     * Wrap a transport as output stream and call
     * t->openWriteDoc(headers) in the constructor.
     */
    otransstream(const Fancy::ObjPtr<Transport> &t, const Fancy::ObjPtr<Map> &headers);
    /**
     * Wrap a transport as output stream and do nothing in the constructor.
     *
     * It is assumed, that t->openWriteDoc(headers) has been called before.
     */
    otransstream(const Fancy::ObjPtr<Transport> &t);
    ~otransstream();
};

}}

#endif
